package info.kwarc.mmt.frameit.rules

import info.kwarc.mmt.api.GlobalName
import info.kwarc.mmt.api.checking.{CheckingCallback, Continue, History, TermBasedEqualityRule}
import info.kwarc.mmt.api.objects.{Equality, Stack, Term}
import info.kwarc.mmt.lf.{Apply, ApplySpine, Typed}

object MetricCommutative extends TermBasedEqualityRule {
  override def applicable(tm1: Term, tm2: Term): Boolean = (tm1,tm2) match {
    case (ApplySpine(Symbols.metric,List(_,_)),ApplySpine(Symbols.metric,List(_,_))) =>
      true // rule is applicable iff both terms are of the form metric(a,b)
    case _ => false
  }

  override def head: GlobalName = Apply.path // not sure if necessary, since applicable-method exists

  override def apply(check: CheckingCallback)(tm1: Term, tm2: Term, tp: Option[Term])(implicit stack: Stack, history: History): Option[Continue[Boolean]] = {
    // destruct terms
    val (ApplySpine(Symbols.metric, List(va1, va2)), ApplySpine(Symbols.metric, List(vb1, vb2))) = (tm1, tm2)
    // check whether va1 == vb1 and va2 == vb2 is trivial and does not require a rule; we only need to check for
    // va1 == vb2 and va2 == vb1
    val iseq = check.tryToCheckWithoutDelay(
      Equality(stack,va1,vb2,None),
      Equality(stack,va2,vb1,None)
    )
    iseq match {
      case Some(true) =>
        Some(Continue(true))
      case _ =>
        None // this rule can not determine equality
    }
  }
}

object AngleInvertible extends TermBasedEqualityRule {
  override def applicable(tm1: Term, tm2: Term): Boolean = (tm1,tm2) match {
    case (ApplySpine(Symbols.angle,List(_,_,_)),ApplySpine(Symbols.angle,List(_,_,_))) =>
      true // rule is applicable iff both terms are of the form angle_between(a,b,c)
    case _ => false
  }

  override def head: GlobalName = Apply.path // not sure if necessary, since applicable-method exists

  override def apply(check: CheckingCallback)(tm1: Term, tm2: Term, tp: Option[Term])(implicit stack: Stack, history: History): Option[Continue[Boolean]] = {
    // destruct terms
    val (ApplySpine(Symbols.angle, List(a1, b1, c1)), ApplySpine(Symbols.metric, List(a2, b2, c2))) = (tm1, tm2)
    // check whether a1 == a1 && b1 == b2 && c1 == c2 is trivial; we only need to check for
    // b1 == b2 && a1 == c2 and c1 == a2
    val iseq = check.tryToCheckWithoutDelay(
      Equality(stack,b1,b2,None),
      Equality(stack,a1,c2,None),
      Equality(stack,c1,a2,None)
    )
    iseq match {
      case Some(true) =>
        Some(Continue(true))
      case _ =>
        None // this rule can not determine equality
    }
  }
}